/*
 * Copyright (C): ООО «Сетевые информационные системы»
 *                http://avreg.net
 *        Author: Андрей Никитин <nik-a@mail.ru>
 */

/**
 * \file
 * \mainpage libavregconf
 *
 * Библиотека libavregconf включает в себя набор функций
 * используемых C-приложениями AVReg avregd и avreg-mon.
 *
 * \n
 * Настоящая версия содержит 2 группы функций:
 *  - \ref avregconf
 *  - \ref dynlist
 */

#ifndef _LIBAVERGCONF_H_
#define _LIBAVERGCONF_H_ 1

#include <syslog.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdint.h>

#include <avregconf/version.h>
#include <avregconf/common.h>
#include <avregconf/str.h>
#include <avregconf/log.h>
#include <avregconf/helpers.h>
#include <avregconf/dyn-list.h>
#include <avregconf/periods.h>
#include <avregconf/misc-utils.h>

/**
 *  @defgroup avregconf Набор функций для парсинга конфигурационного файла avreg.conf.
 *
 *  Структура файла avreg.conf
 *  \verbatim
    # Строки, начинающиеся с символов "#" или ";" являются
    ;  комментариями и игнорируются

    # Глобальная секция - общие параметры для всех приложений
    global_par1 = value
    global_par2 = "other value"

    # секция параметров приложения app1
    app1 {
       ...
       app1_par1 = value
       ...
    }

    # секция параметров приложения app2
    app2 {
       ; подключить дополнительный файл конфигурации
       include "/path/app2.conf"
       app2_par1 = value
       ...
    }
    \endverbatim
 *
 * @{
 */

/**
 * Имя профиля - дополнительного конфигурационного файла.
 */
const char *avregconf_get_avreg_profile(void);
void avregconf_set_avreg_profile(const char *avreg_profile_name);

/**
 * Каллбэк функция, принимающая имя и значение параметра,
 * вычитываемые из avreg.conf.
 *
 * @param[in] opaque - ссылка на приватные данные клиентского процесса (см. avregconf_parse(..., opaque)).
 * @param[in] file - имя файла конфигурации.
 * @param[in] lineno - номер строки.
 * @param[in] param - имя параметра.
 * @param[in] value - значение параметра.
 *
 * @return AVREGPARSE_SUCCESS или AVREGPARSE_FAILED.
 */
typedef int(* avregconf_callback_t)( void *opaque,
                                     const char *file,
                                     int lineno,
                                     const char *param,
                                     const char *value );
/*
 * Группировка параметров { ... }
 *
 * @note Не реализовано польностью, не используется avregd 5.x avreg-mon 5.x
 */
typedef int(* avregconf_pargroup_enter_cb_t)( void *opaque,
                                              const char *file,
                                              int lineno,
                                              int  pargroup_depth,
                                              const char *pargroup,
                                              const char *value );
typedef int(* avregconf_pargroup_leave_cb_t)( void *opaque,
                                              const char *file,
                                              int lineno,
                                              int  pargroup_depth );

typedef int(* avregconf_newparval_cb_t)( void *opaque,
                                         const char *file,
                                         int lineno,
                                         const char *param,
                                         const char *value );

typedef struct avregconf_parse_pargroup_s {
   int min_depth;
   int max_depth;
   avregconf_newparval_cb_t parse;
   avregconf_newparval_cb_t parse_pgr_value;
} avregconf_parse_pargroup_t;

/**
 * Читает конфигурационный файл и передаёт param=value пользовательскую cbk_func.
 *
 * @param[in] conf_name - путь до конфигурационного файла.
 * @param[in] program_name - имя программы, используется при логгировании.
 * @param[in] cbk_func - пользовательская каллбэк-функция, которой передаются найденные param=value.
 * @param[in] max_include_recursion - макс. уровень вложений include /path/subconf_file
 * @param[in] opaque - указатель на приватные данные приложения, передаётся в cbk_func()
 *
 * @return AVREGPARSE_SUCCESS или AVREGPARSE_FAILED.
 */
int
avregconf_parse(const char *conf_name,
              const char *program_name,
              avregconf_callback_t cbk_func,
              int max_include_recursion,
              void *opaque);

/* новая ф-ция с поддержкой вложенных груп, не реализовано до конца*/
int
avregconf_parse2(const char *conf_name,
              const char *program_name,
              avregconf_pargroup_enter_cb_t pargroup_enter_cbk,
              avregconf_pargroup_leave_cb_t pargroup_leave_cbk,
              avregconf_newparval_cb_t   parval_cbk,
              int max_include_recursion,
              void *opaque);

/* other functions */

/** @} */ // end of group avregconf

#endif /* _LIBAVERGCONF_H_ */
