/*
 * daemonize.h
 *
 *   (c) 2016 Andrew Nikitin <nik-a@mail.ru>
 *
 */
#ifndef INCLUDE_AVREGCONF_DAEMONIZE_H
#define INCLUDE_AVREGCONF_DAEMONIZE_H

/***
 * Если задан, то закрываются только STDIN и STDERR
 */
#define AVREG_DAEMONIZE_FLAG_ESCAPE_CLOSE   1

/***
 * Если задан, то выполняется openlog(program_name, LOG_NOWAIT, LOG_DAEMON)
 */
#define AVREG_DAEMONIZE_FLAG_ESCAPE_OPENLOG 2

/***
 * Демонизирует процесс.
 *
 * Алгоритм работы функции:
 *   - прроверяются вх. параметры,
 *   - umask(0),
 *   - avreg_create_pidfile()
 *   - pipe(control_pipe) для обеспечения таймаута запуска и гарантии успешности запуска рабочего процесса,
 *   - sigaction(SIGCHLD, SIG_IGN)
 *   - fork() - если ошибка, если ошибка то exit(EXIT_FAILURE)
 *
 *   child proccess - ноый рабочий процесс демона
 *   - fclose(pidfile)
 *   - setsid()
 *   - if (user || groups) { set_rights(pwn_rec, grp_rec); set_core_dumpable(max_core_sz))
 *   - if (user) { setenv(LOGNAME, USER, USERNAME, HOME, MAIL) }
 *   - chdir(rootdir || "/")
 *   - if (!donot_close_fds) close(opened_descriptors), см. также AVREG_DAEMONIZE_FLAG_ESCAPE_CLOSE
 *   - stdin = stdout = strerr = /dev/null
 *   - if (AVREG_DAEMONIZE_FLAG_ESCAPE_OPENLOG) { openlog(program_name, LOG_NOWAIT, LOG_DAEMON) }
 *   - close(control_pipe_read_end_fd)
 *   - return (-1 если ошибка || control_pipe_write_end)
 *
 *   parent process - текущий процесс
 *   - close(control_pipe_write_end)
 *   - select(control_pipe_read_end_fd, start_timeout)
 *     ожидается текстовое сообщ. "OK",
 *     если вернулся 0 (рабочий процесс оборвал pipe) или таймаут,
 *     то удаляется pidfile и процесс прерывается exit(EXIT_FAILURE);
 *     если child вернул OK, то 
 *        fprintf(pidf, "%d\n", child_pid)
 *        fchown(pidfile) если заданы user||groups
 *        close(pidfile)
 *        close(control_pipe_read_end_fd)
 *
 *  Порядок использования фунции:
 *
 *    # parent proc: init0_stage_before_fork
 *    ...
 *    parentPipeFd = avreg_daemonize()
 *    if (0 > parentPipeFd) {
 *       goto startup_failure;
 *    }
 *    
 *    # child daemon proc: init1_stage_after_fork
 *    ...
 *    
 *    if (init1_stage_after_fork_is_OK) {
 *       write(parentPipeFd, "OK");
 *    }
 *    close(parentPipeFd);
 *    
 *    main_loop()
 *
 *
 * param[in] program_name Имя процесса (basename)
 * param[in] pid_name Pid-файл, если не задан (NULL), то будет использоваться {PID_DIR}/{program_name}.pid
 * param[in] start_timeout Таймаут ожидания секции инициализации рабочего процесса
 * param[in] user Имя пользователя для выполнения рабочего процесса, м.б. NULL
 * param[in] group Имя группы для выполнения рабочего процесса, м.б. NULL
 * param[in] rootdir Корневой каталог рабочего процесса, если не задан (NULL) используется "/"
 * param[in] donot_close_fds Не закрывать открытые до fork(2) файловые дескрипторы.
 * param[in] max_core_sz Размер coredump в байтах. В теории 0 - должен запрещать coredump.
 *                       Используется только если заданы user || groups (не знаю почему, лень смотреть :)
 * param[in] flags Флаги AVREG_DAEMONIZE_FLAG_XXX
 * @return -1 если ошибка или control_pipe_write_end, для записи "OK" после успешной init-фазы нового процесса-демона
 */
int avreg_daemonize(
      const char *program_name,
      const char *pid_name,
      int start_timeout,
      const char *user,
      const char *group,
      const char *rootdir,
      const int  *donot_close_fds,
      unsigned long max_core_sz,
      int flags
);

#endif // INCLUDE_AVREGCONF_DAEMONIZE_H
