#ifndef INCLUDE_AVREGCONF_DYN_LIST_H
#define INCLUDE_AVREGCONF_DYN_LIST_H 1

/**
 *  @defgroup dynlist  Набор функций для работы с динамическим списком.
 *
 *  Динамический список представляет собой неограниченный количеством элементов
 *  массив. Каждый элемент несёт в себе объект данных и ссылки на соседние элементы.
 *
 *  Рассматриваемые функции предназначены для убодного обхода списка,
 *  поиска по шаблонному объекту и каллбек-обработчиков на добавление 
 *  и удаление элементов из списка.
 *  
 *  @{
 */

/**
 * Ошибка возвращаемая функциями динамического листа.
 */
#define DLST_FAIL    0
/**
 * Успешное выполнение функций динамического листа.
 */
#define DLST_SUCCESS 1

/**
 * Допустимые флаги flags в dlst_init()
 */

/***
 * Использовать блокировки на основе мьютексов для обеcпечения безопасного доступа
 * к списку функциями с суффиксом_r из разных потоков.
 */
#define DLST_FLAG_CREATE_MUTEX 0x01

/***
 * Устанавливает флаг dlst->sorted 
 */
#define DLST_FLAG_SORTED       0x02

/**
 * Тело (данные) элемента списка DYN_LIST.
 */
typedef  void  DLST_BODY;
/**
 * Ссылка на тело (данные) элемента списка DYN_LIST.
 */
typedef  DLST_BODY* DLST_BODY_P;

/**
 * Тип структуры DYN_LIST
 */
typedef struct dlst_desc DYN_LIST;

/**
 * Элемент динамического списка DYN_LIST.
 */
typedef struct dlst_elem {
   DLST_BODY  *body;        ///< указатель на тело элемента (данные)
   struct dlst_elem *next;  ///< ссылка на следующий элемент или NULL если последний
   struct dlst_elem *prev;  ///< ссылка на предыдущий элемент или NULL если первый

   DYN_LIST *owner_dlst;    ///< ссылка на родительский DYN_LIST
} DLST_ELEM;

/**
 * Структура DYN_LIST
 */
struct dlst_desc {
    char  *short_list_name;      ///< краткое имя списка, используется при логгировании
    unsigned int  nr;            ///< число элементов в списке

    DLST_ELEM *head;             ///< ссылка на первый элемент
    DLST_ELEM *tail;             ///< ссылка на последний элемент

    size_t body_sz;              ///< размер данных #DLST_BODY элемента #DLST_ELEM, 
                                 ///< !!! постоянное значение в пределах одного DYN_LIST

    void (*on_elem_remove) ();   ///< обработчик, вызываемый при удалении элемента из списка

    /**
     * Функция сравнения #DLST_BODY
     *
     * Должена возвращать -1(меньше), 1(больше) и 0(совпадает) аналогично strcmp(3).
     *
     * Используется в ф-циях поиска и dlst_append_uniq().
     */
    int  (*body_cmp) ();

    /**
     * Обработчик, вызываемый при копировании тела элемента при добавлении.
     *
     * В deprecated dlst_append() обработчик body_copy() (если установлен)
     * вызывается после копирования (memcpy) тела #DLST_BODY исходного в "свежевыделенный" #DLST_BODY_P
     *
     * В новой dlst_append2() будет вызван обработчик body_copy() если он установлен
     * или memcpy((void *)new_elem->body, (void *)body, dlst->body_sz);
     */
    int  (*body_copy) ();

    void    *mutex; ///< ссылка на мьютекс, который возможно будет использоваться для разделения доступа

    int sorted; ///< при добавлении массива строит отсортированный список в нормальном порядке,
                ///< по возрастанию, cм. flags dlst_init()
};

struct voidp {
   void *p;
};

/**
 * Тип функции, вызываемой при удалении элемента из списка.
 */
typedef void   (* on_elem_remove_t) ( DLST_BODY_P body );

#define INT_CMP_RETURN(a,b) if ( a != b ) return a > b ? 1 : -1
 /**
 * Функция сравнения #DLST_BODY
 *
 * Должена возвращать -1(меньше), 1(больше) и 0(совпадает) аналогично strcmp(3).
 *
 * Используется в ф-циях поиска и dlst_append_uniq().
 */
typedef int(* body_cmp_t) (const DLST_BODY_P wanted_body, const DLST_BODY_P list_body);

/**
 * Тип функции, вызываемой при копировании тела элемента при добавлении.
 *
 * Должен возвращать -1(меньше), 1(больше) и 0(совпадает) аналогично strcmp(3).
 *
 * В deprecated dlst_append() обработчик body_copy() (если установлен)
 * вызывается после копирования (memcpy) тела #DLST_BODY исходного в "свежевыделенный" #DLST_BODY_P
 *
 * В новой dlst_append2() будет вызван обработчик body_copy() если он установлен
 * или memcpy((void *)new_elem->body, (void *)body, dlst->body_sz);
 */
typedef int(* body_copy_t) (DLST_BODY_P dst_body, const DLST_BODY_P src_body);

/**
 * Создание списка.
 *
 * @param[in] elem_body_sz,on_elem_add,on_elem_remove,body_cmp,body_copy,short_list_name
 *                            - см. одноимённый поля структуры #DYN_LIST.
 * @param[in] flags - набор флагов DLST_FLAG_XXX

 * @return  DYN_LIST* или NULL (+errno) при ошибке.                        
 */
DYN_LIST   *dlst_init ( size_t  	        elem_body_sz,
                        on_elem_remove_t on_elem_remove,
                        body_cmp_t	     body_cmp,
                        body_copy_t	     body_copy,
                        const char       *short_list_name,
                        int              flags);

/**
 * По-очереди удаляет все элементы из списка с вызовом обработчика on_elem_remove()
 * и освобождает ресурсы выделенные под список.
 *
 * dlst_{clean,free}* -  реентрабильные функции без суффикса _r,
 * использующие блокировку если при создании списка был указан флаг DLST_FLAG_CREATE_MUTEX
 *
 * dlst_clean*() - удаляют все элементы из списка с вызовом обработчика on_elem_remove()
 * dlst_free*() - dlst_clean() + удаляют сам список.
 * dlst_*_backward() - с обратным обходом(удалением) элементов с конца списка.
 *
 * Флаг do_not_lock запрещает блокировку, если список был создан с использованием механизмов
 * блокировки (с мьютексом).
 */

int   dlst_clean(DYN_LIST *dlst, int do_not_lock);
int   dlst_clean_backward(DYN_LIST *dlst, int do_not_lock);
int   dlst_free   (DYN_LIST **dlst_p);
int   dlst_free_backward (DYN_LIST **dlst_p);

/* ф-ции для "ручной" блокировки/разблокировки списка мьютексами */
int   dlst_lock   (const DYN_LIST *dlst);
int   dlst_unlock (const DYN_LIST *dlst);

/**
 * Добавляет объект (тело, данные) представленный ссылкой void* в конец списка.
 *
 * Сначала выделяется память malloc(elem_body_sz) под тело(данные) объекта.
 * Затем, если при создании списка обработчик body_copy() не был установлен,
 * данные объекта просто переписываются memcpy(). В противном случае вызывается body_copy().
 *
 * В элемент списка #DLST_ELEM записывается указатель на только что выделенную память,
 * а не исходный body.
 *
 * Если #DYN_LIST->elem_body_sz = 0, то предполагается что объект без данных,
 * просто указатель. В этом случае память не выделяется и ничего не копируется.
 * 
 * @return прямая ссылка на DLST_ELEM внутри списка или NULL(+errno) при ошибке.
 */
DLST_ELEM  *dlst_append (DYN_LIST *dlst, const DLST_BODY_P body);

/***
 * Реентрабильная версия dlst_append()
 * @return DLST_FAIL или DLST_SUCCESS
 */
int dlst_append_r (DYN_LIST *dlst, const DLST_BODY_P body);

/**
 * Действует аналогично dlst_append(),
 * только если элемент с похожим объектом на основаниис body_cmp() существует,
 * то если задано fail_if_duplicate возвращается ссылка на него вместо создания нового
 * или возвращается NULL.
 *
 * @return прямая ссылка на DLST_ELEM внутри списка или NULL(+errno) при ошибке.
 */
DLST_ELEM  *dlst_append_uniq   (DYN_LIST *dlst, const DLST_BODY_P body, int fail_if_duplicate, body_cmp_t other_cmp_func);

/**
 * Реетрабильная версия dlst_append_uniq()
 *  @return DLST_SUCCESS или DLST_FAIL.
 */
int dlst_append_uniq_r (DYN_LIST *dlst, const DLST_BODY_P body, int fail_if_duplicate, body_cmp_t other_cmp_func);


/**
 * Удаляет #DLST_ELEM из списка и возвращает ссылку на следующий #DLST_ELEM в списке,
 * после удалённого #DLST_ELEM.
 */
DLST_ELEM  *dlst_remove_and_get_next (DYN_LIST *dlst, DLST_ELEM *elem);

/**
 * Поиск похожего элемента на основании штатной функции сравнения #DYN_LIST->body_cmp()
 * или other_cmp_func если задано.
 */
DLST_ELEM  *dlst_find (const DYN_LIST *dlst, const DLST_BODY_P wanted_body, body_cmp_t other_cmp_func);

/**
 * Реентрабильная версия dlst_find()
 * 
 * @param[in] dlst - указатель на список;
 * @param[in] wanted_body - ссылка на объект для поиска, сравнение по ф-ции other_cmp_func() или #DYN_LIST->body_cmp();
 * @param[out] result_body - если не NULL, сюда копируется тело найденного объекта;
 * @param[in] other_cmp_func - пользовательская ф-ция сравнения объектов.
 *
 * @return DLST_FAIL(не найдено) или DLST_SUCCESS(найдено)
 */
int dlst_find_r (const DYN_LIST *dlst, const  DLST_BODY_P wanted_body, body_cmp_t other_cmp_func, DLST_BODY_P result_body);

/**
 * Возвращает ссылку #DLST_ELEM, объектом данных которого является передаваемый body.
 */
DLST_ELEM  *elemsbody2elem           (const DYN_LIST *dlst, const DLST_BODY_P body);

/**
 * Тип каллбэк-функции для dlst_each()
 */
typedef int (* for_each_cbk_t )  (const DLST_ELEM *elem, void *opaque);

/**
 * Реентрабильный итератор.
 *
 * Обходит список, для каждого элемента вызывает каллбэк-функцию cbk,
 * которой передаёт ссылку на #DLST_ELEM и opaque.
 *
 * Использует  dlst_lock()/dlst_unlock()
 *
 * Если каллбэк-функция cbk возвратит DLST_FAIL,
 * текущий #DLST_ELEM будет удалён из списка,
 * а обход продолжится далее.
 */
void dlst_each_r (DYN_LIST *dlst, for_each_cbk_t cbk, void *opaque);

/**
 * Тип каллбэк-функции для dlst_each2()
 */
typedef int (* for_each_cbk2_t )  (const DLST_BODY_P body, void *opaque, void *opaque2);
#define DLST_ITER_DONOT_REMOVE  0x1  ///< не удалять #DLST_ELEM если ckb() возвратит DLST_FAIL
#define DLST_ITER_BREAK_ON_FAIL 0x2  ///< прекратить обход если ckb() возвратит DLST_FAIL
#define DLST_BACKWARD_WALK      0x4  ///< обратный обход

/**
 * Реентрабильный итератор, версия2.
 *
 * Обходит список, для каждого элемента вызывает каллбэк-функцию cbk,
 * которой передаёт ссылку на #DLST_ELEM, opaque и opaque2.
 *
 * Использует  dlst_lock()/dlst_unlock()
 *
 * Если каллбэк-функция cbk возвратит DLST_FAIL,
 * текущий #DLST_ELEM будет удалён из списка,
 * а обход продолжится далее.
 *
 * Допустимые флаги параметра flags см. define EACH_* выше.
 *
 * \return 0 - если не было ошибок или количество сбойных вызовов  ckb()
 */
int dlst_each2_r (DYN_LIST *dlst, for_each_cbk2_t cbk, int flags, void *opaque, void *opaque2);

/***
 * Удаляет элемент из списка. Реентрабильная версия.
 * @return DLST_SUCCESS (даже если элемент не был найдён) или DLST_FAIL (только при сбое в dlst_success)
 */
int dlst_remove_r (DYN_LIST *dlst, DLST_ELEM *elem);

/**
 * XXX Если нужен потокобезопасный аналог dlst_remove_and_get_next(),
 * кодируйте примерно так:
 * \code
 * if ( DLST_FAIL != dlst_lock() ) {
 *   DLST_ELEM  *elem = dlst_find(...);
 *   elem = dlst_remove_and_get_next(elem);
 *   dlst_unlock();
 * }
 * \endcode
 */

/**
 * Ищет элемент по данным объекта и удаляет его (элемент)
 */
int  dlst_remove_by_body_r (DYN_LIST *dlst, const  DLST_BODY_P body);

/* вывод содержимого листа и елемента в лог для отладочных целей */
void dlst_print_list (const DYN_LIST  *dlst);
void dlst_print_elem (const DLST_ELEM *elem);
/** @} */ // end of group dynlist

#endif
