#ifndef INCLUDE_AVREGCONF_HELPERS_H
#define INCLUDE_AVREGCONF_HELPERS_H 1

#include <stdint.h>

#include "avregconf/common.h"
#include "avregconf/log.h"

/**
 * Элемент простого строкового словаря, последний элемент которого (хэша) должен быть NULL
 * Пример хеша:
 * \code
 * avregconf_dict_elem_t h[] = {
 *    { "abc", 1, p1 },
 *    { "def", 2, p2 },
 *    { NULL },
 * }
 * \endcode
 */
typedef struct avregconf_dict_elem_s {
   char *key; ///< строка
   int    id; ///< числовой идентификатор
   void *ptr; ///< произвольный указатель
} avregconf_dict_elem_t;

/**
 * Элемент простого хеша
 * Пример хеша:
 * \code
 * static const hash_elem_t avreg_protos_hash[] = {
 *   { AVREG_PROTO_FILE,      FILE_S,  4 },
 *   { AVREG_PROTO_PIPE,      PIPE_S,  4 },
 *   { AVREG_PROTO_V4L,       V4L_S ,  3 },
 *   { AVREG_PROTO_V4L,       V4L2_S,  4 },
 *   { AVREG_PROTO_V4L,       "video4linux",  11 },
 *   { AVREG_PROTO_HTTP,      HTTP_S,  4 },
 *   { AVREG_PROTO_RTSP,      RTSP_S,  4 },
 *   { AVREG_PROTO_ALSA,      ALSA_S,  4 },
 *   { AVREG_PROTO_UNDEFINED,  NULL ,  0 },
 * }
 * \endcode
 */
typedef struct avregconf_hash_elem_s {
    int         id;
    const char  *key;
    size_t      key_len;
} avregconf_hash_elem_t;

typedef struct avregconf_u32flags_s {
   unsigned int mask;
   char *str;
} avregconf_u32flags_t;

/**
 * Поиск элемента в простом строковом хэше.
 */
const avregconf_dict_elem_t *
avregconf_dict_find(const avregconf_dict_elem_t dict[], const char *key, int caseignore);

const avregconf_hash_elem_t *
avregconf_hash_find(const avregconf_hash_elem_t hash[], const char *key, int case_ignore);

/**
 * Анализирует "булевую" строку.
 *
 * @return
 *   - AVREGPARSE_TRUE если строка начинается с символов [1yYtT].
 *   - AVREGPARSE_FALSE для других символов
 */
int
avregconf_parse_bool (const char *string);

/**
 * Быстрая сортировка Хоара массива целых чисел.
 */
void avregconf_hsort (int arr[], int l, int r);
/**
 * Удаляет дубликаты из массива целых чисел.
 */
int avregconf_uniq  (int arr[], int l, int r);

/**
 * Анализирует строку - номерной список и формирует массив целых чисел входящий в исходный список.
 *
 * @param[in] src - список номеров, например "1,5,4-7,4"
 * @param[out] dst - ссылка на int-массив, в который сохраняются номера.
 * @param[in] dst_nmemb - размер массива dst в элементах. 
 * @return - кол-во int, сохранённых в dst, при ошибках устанавливается errno.
 */
unsigned int
avregconf_parse_numlist(const char *src, int dst[], unsigned int dst_nmemb);

/* новая версия avregconf_parse_numlist() - не используется пока */
typedef unsigned int(* avregconf_numlist_error_cb_t)(void *opaque, const char *chunk, int **append);
unsigned int
avregconf_parse_numlist2(void *opaque, const char *src,
                         unsigned int dst_nmemb, int num_min, int num_max,
                         avregconf_numlist_error_cb_t on_error,
                         int dst[]);

/* return -1 on error */
int avregconf_parse_decimal_unsigned(const char *line, int max_chars_before_digit);


/*
   char tmp[256];
   char *p = argv[1];
   size_t token_len;
   while ( NULL != (p = avregconf_str_get_next_token(p, ", ;\t\r", &token_len))) {
      int a = MIN(sizeof(tmp)-1, token_len);
      strncpy(tmp,p,a);
      tmp[a] = '\0';
      printf("found (%d) \"%s\"\n",token_len, tmp);
      p += token_len;
   }
*/
char *avregconf_str_get_next_token(const char *line, const char delims[], size_t *token_len);

#define AVREGCONF_FLAGS_DELIMS_DEF ", ;+\t\r"
int avregconf_parse_flags(
      const avregconf_dict_elem_t flags_dict[],
      const char *flags_str,
      const char *delims,
      uint32_t *flags);

int avregconf_in_array_int(const int *ai, unsigned int ai_nb, int val);
// array of int to csv
int avregconf_implode_int(int arr[], int a_sz, char *dst, size_t dst_sz);

char *avregconf_u32flag2str_r(const avregconf_u32flags_t flags[], uint32_t value, char *out, size_t outsz);

char *avregconf_rand_string(char *str, size_t size, const char charset[]);

/*
 * parse lines:
 * streamtype=5; profile-level-id=15; config=1588; 
 * a:b ; c:d; blabla; xxx= 33   ; dd
 */
#define AVREGCONF_FIELDS_SEPARATORS_DEF ";\n\t\r"
#define AVREGCONF_TOKENS_SEPARATORS_DEF "=:"
char *
avregconf_next_parval_pair(
   char *lines,
   const char fields_sep[],
   const char tokens_sep[],
   char **par_start_char,
   char **val_start_char
);

#endif
