#ifndef INCLUDE_AVREGCONF_NET_H
#define INCLUDE_AVREGCONF_NET_H 1

/**
 * @file
 * libaregconf network utility public API header
 */

#include <net/if.h>
#include <netinet/in.h>

#include <avregconf/dyn-list.h>

#define AGA_INET_V4   0x0001
#define AGA_INET_V6   0x0002
#define AGA_INET_ANY  0x0003

#define AGA_NODE_IP       0x0010
#define AGA_NODE_HOSTNAME 0x0020
#define AGA_NODE_IFNAME   0x0040
#define AGA_NODE_ASTERISK 0x0080
#define AGA_NODE_ANY      0x00F0

#define AGA_ONLY_LOCAL    0x1000

// FIXME
#define IN6ADDR_NONE_INIT { { { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 } } }

typedef struct l2addr_s {
   unsigned int len;
   unsigned char addr[IFHWADDRLEN];
} l2addr_t;

typedef struct __ip_addr_s {
   struct in_addr v4;
   struct in6_addr v6;
} __ip_addr_t;

typedef struct __if_link_s {
   /***
    * device_type if_arp.h
    * ARPHRD_ETHER 	     1   Ethernet 10/100Mbps.
    * ARPHRD_EETHER	     2   Experimental Ethernet
    * ARPHRD_IEEE802	     6   IEEE 802.2 Ethernet/TR/TB
    * ARPHRD_LOOPBACK	772
    */
   unsigned short device_type;
   unsigned int flags;

   /* typs is an IFLA_LINK value.
    * For usual devices it is equal ifi_index.
    * If it is a "virtual interface" (f.e. tunnel), ifi_link
    * can point to real physical interface (f.e. for bandwidth calculations),
    * or maybe 0, what means, that real media is unknown (usual
    * for IPIP tunnels, when route to endpoint is allowed to change)
    */
   unsigned int type;
   unsigned int mtu;
   l2addr_t ha;
   l2addr_t ha_bcast;
} __if_link_t;

typedef struct __if_addr_s {
   char label[IF_NAMESIZE];
   unsigned char prefixlen;
   uint32_t flags;
   unsigned char scope;
   struct {
      __ip_addr_t interface;
      __ip_addr_t local;
      __ip_addr_t broadcast;
      __ip_addr_t anycast;
   } addr;      
} __if_addr_t;

typedef struct __if_info_s {
   int index;
   char name[IF_NAMESIZE];
   __if_link_t link;
   __if_addr_t *addr; // pointer to array of __if_addr_t
   unsigned int addr_count;
} __if_info_t;

/***
 * The resulting array will be used to find the addresses assigned to interfaces
 * in the following functions:
 *   - avregconf_getaddr()
 *
 * avregconf_get_iflist() must be called before call of these functions.
 *
 * On shared library case,
 * avregconf_get_iflist() and avregconf_reload_iflist() invoked automatically by dlopen(),
 *
 * @return DYN_LIST of __if_info_t on success, NULL on error.
 *
 * Multithread safe!
 */
const DYN_LIST *avregconf_get_iflist(void);
const DYN_LIST *avregconf_reload_iflist(void);
int if_cmp_by_l2addr (const DLST_BODY_P wanted_body, const DLST_BODY_P list_body);
int if_cmp_by_addr (const DLST_BODY_P wanted_body, const DLST_BODY_P list_body);

/***
 * Get binary address of UP interface or IP string or hostname.
 *
 * @param[in]  node        Name of local interface or IP string or hostname
 * @paramp[in] aga_flags   Set of AGA_XXX_XXX flags (see above)
 * @param[out] addrbuf     Array of struct sockaddr for result, might be NULL
 * @param[in]  addrbuf_nbr Addrbuf length in (struct sockaddr), might be 0
 * @param[out] errsrc      Buffer to copy error message.
 * @param[in]  errsrc_sz   Size of errsrc buffer.
 * @return                 < 0 - error, return getaddrinfo(3) return value.
 *                         = 0 - error if AGA_ONLY_LOCAL flag used,
 *                               adress or ip into node value valid,
 *                               but not local.
 *                         > 0 - success, return total address number
 *                               (can be greater than addrbuf_nbr).
 */
int avregconf_getaddr(const char *node, int aga_flags, struct sockaddr_storage *addrbuf, int addrbuf_nbr,
      char *errsrc, size_t errsrc_sz);

int
avregconf_parse_ip4_acl(const char *node,
                     struct in_addr *ipv4_in_addr,
                     struct in_addr *mask4_in_addr);
int
avregconf_ipv4_cmp(const struct in_addr a1, const struct in_addr m1,
                   const struct in_addr a2, const struct in_addr m2);
 /* 255.255.255.255 = INADDR_NONE ((in_addr_t) 0xffffffff) */

#endif /* INCLUDE_AVREGCONF_NET_H */
